TRACKOMATIC_API = {};
TRACKOMATIC_API.__index = TRACKOMATIC_API;

local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

function TrackOMatic_GetPluginAPI(pluginId, interfaceVersion)
    local api = {};
    setmetatable(api, TRACKOMATIC_API);
    api.messageColor = {r = 1, g = 1, b = 1};
    api.pluginId = pluginId;
    api.interfaceVersion = interfaceVersion;
    api.registered = false;
    return api;
end

function TRACKOMATIC_API:RegisterPlugin()
    if (self.registered) then return; end
    if ((not self.interfaceVersion) or (self.interfaceVersion ~= TRACKOMATIC_PLUGIN_INTERFACE)) then
        TRACKOMATIC_BAD_PLUGINS[self.pluginId] = {id = self.pluginId, name = self.name, version = self.version, author = self.author, why = L["INCOMPATIBLE"]};
        TrackOMatic_Message(string.format(L["OUTDATED_PLUGIN"], self.name));
        return;
    end
    if ((not TRACKOMATIC_PLUGINS[self.pluginId]) and (not TRACKOMATIC_BAD_PLUGINS[self.pluginId])) then
        TRACKOMATIC_PLUGINS[self.pluginId] = {id = self.pluginId, name = self.name, version = self.version, author = self.author, category = self.category, onUpdate = self.onUpdate, onClick = self.onClick, onAdd = self.onAdd, onRemove = self.onRemove};
        self.msgClr = self.messageColor;
        if (not TRACKOMATIC_VARS['config']['silent_startup']) then
            TrackOMatic_Message(string.format(L["REGISTERED_PLUGIN"], self.name .. " [" .. self.version .. "]"));
        end
        if (not self.onUpdate) then
            TrackOMatic_Message(string.format(L["PLUGIN_NULL_FUNCTION"], self.name .. " [" .. self.version .. "]"));
        end
        self.registered = true;
    end
end

function TRACKOMATIC_API:SetData(key, value)
    if (not self.registered) then return; end
    if (type(key) ~= "string") then return; end
    if (not TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['plugin_data'][self.pluginId]) then
        TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['plugin_data'][self.pluginId] = {};
    end
    TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['plugin_data'][self.pluginId][key] = value;
end

function TRACKOMATIC_API:GetData()
    if (not self.registered) then return {}; end
    return (TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['plugin_data'][self.pluginId] or {});
end

function TRACKOMATIC_API:GetValue(key)
    if (not self.registered) then return {}; end
    if (TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['plugin_data'][self.pluginId]) then
        return TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['plugin_data'][self.pluginId][key];
    end
end

function TRACKOMATIC_API:Message(text)
    if (not self.registered) then return; end
    DEFAULT_CHAT_FRAME:AddMessage(TrackOMatic_ColorRGBToHex(self.msgClr) .. "<" .. self.name .. ">|r " .. text);
end

function TRACKOMATIC_API:UpdateTracker()
    if (not self.registered) then return; end
    TrackOMatic_UpdateTracker();
end

function TRACKOMATIC_API:RebuildTracker()
    if (not self.registered) then return; end
    TrackOMatic_BuildTracker();
end

function TRACKOMATIC_API:CommonMenuItem(id, func, ...)
    if (not self.registered) then return nil; end
    return TrackOMatic_CommonMenuItem(id, func, ...);
end
